#!/usr/bin/env python
""" This script is to calculate the HPFs fiber and CFB positions on HET telescope.
If something is not clear, feel free to drop me an email - Joe Ninan, indiajoe@gmail.com """

from __future__ import division
import sys
import argparse
import numpy as np
from scipy.optimize import leastsq
import matplotlib.pyplot as plt
from collections import OrderedDict
##########################################################################
# Library of functions we will use later in this calculation.
######################################################
### Calculation of general purpose transformation matrix
#### These are constrained transformation matrix where scaling in x and y is same.

def ScaleMatrix(Scale):
    ScaleMatrix = np.matrix([[Scale,0,0],
                             [0,Scale,0],
                             [0,0,1]])
    return ScaleMatrix

def TranslateMatrix(TranX,TranY):
    TranslateMatrix = np.matrix([[1,0,TranX],
                                 [0,1,TranY],
                                 [0,0,1]])
    return TranslateMatrix

def RotMatrix(theta):
    RotMatrix = np.matrix([[np.cos(theta),-np.sin(theta),0],
                           [np.sin(theta),np.cos(theta),0],
                           [0,0,1]])
    return RotMatrix

def ReflectYMatrix():
    ReflectYMatrix = np.matrix([[1,0,0],
                                [0,-1,0],
                                [0,0,1]])
    return ReflectYMatrix

def ReflectXMatrix():
    ReflectXMatrix = np.matrix([[-1,0,0],
                                [0,1,0],
                                [0,0,1]])
    return ReflectXMatrix

###################################
# Error functions to fit the pramaeters of transformation matrix

# Function to fit the Translate*Rotate*Scale transformation
def error_funcTRS(p,OldCoords,NewCoords):
    """ Error function to fit the coordinate where parameters are p=[Scale,theta,TranX,TranY]
    which will transform OldCoords to NewCoords
    Fitted transformation is TRS: Translate*Rotate*Scale """
    RS = np.matmul(RotMatrix(p[1]),ScaleMatrix(p[0]))
    TRS = np.matmul(TranslateMatrix(p[2],p[3]),RS)
    TransformCoords = np.matmul(TRS,OldCoords)
    return np.array(TransformCoords.flatten() - NewCoords.flatten())[0]

# Function to fit the Translate*Rotate*Scale*ParityFlipY transformation
# Not used for naything now. Just here for completeness
def error_funcTRSP(p,OldCoords,NewCoords):
    """ Error function to fit the coordinate where parameters are p=[Scale,theta,TranX,TranY]
    which will transform OldCoords to NewCoords
    Fitted transformation is TRSP: Translate*Rotate*Scale*ParityFlipY"""
    SP = np.matmul(ScaleMatrix(p[0]),ReflectYMatrix())
    RSP = np.matmul(RotMatrix(p[1]),SP)
    TRSP = np.matmul(TranslateMatrix(p[2],p[3]),RSP)
    TransformCoords = np.matmul(TRSP,OldCoords)
    return np.array(TransformCoords.flatten() - NewCoords.flatten())[0]

#############################################################################


# There are three Scenarios of calculation

#HPF fiber head fiber positions in telecentric lens images measured at PSU lab.
CentroidCoords = OrderedDict()
CentroidCoords['cCFB'] = (641.7590850208674, 378.2411130588114)
CentroidCoords['sCFB'] = (373.99530838383396, 644.5011001356886)
CentroidCoords['oct1'] = (241.7727067933783, 715.9757709056951)
CentroidCoords['oct2'] = (298.5672272093679, 773.1045874545131)
CentroidCoords['oct3'] = (717.8617085355628, 248.97764755048598)
CentroidCoords['oct4'] = (774.8631863343908, 305.7149638147935)
CentroidCoords['cir1'] = (222.25408901265263, 333.5099840115954)
CentroidCoords['cir2'] = (273.45298734501097, 270.7847529387491)
CentroidCoords['cir3'] = (336.04330942778, 222.2177477751225)
CentroidCoords['cir4'] = (680.1842052725259, 800.6668579582121)
CentroidCoords['cir5'] = (744.1546788398894, 752.2047326301424)
CentroidCoords['cir6'] = (793.7851982362772, 688.9766017329656)

def Calculate_all_coordinates_from_scratch(DictOfCoords,MakePlot=False):
    """ 
    Scenrio 1
    Calculates all the coordinates from scratch using new measured positions
    ## Scenario 1: Everything from scratch ############################################
    
    # When: This is when you suspect the HPF's fiber puck inserted in the HET's focal plane has moved with respect to BIB, LRS2 etc..
    
    # Things to do: 
    # 1. Measure the cCFB and sCFB positions in in ACAM
    # 2. Calculate all of the HPF fiber positions in the fplane.txt file
    # 3. Calculate the offsets in X & Y coordinates for moving from cCFB to science fibers.
    # 4. Calculate the ACAM pixel coordinates for all the fibers.
    
    ### Step 1: Measure the cCFB and sCFB positions in ACAM
    
    # 1. Use the previous estimate of the ACAM coordaintes to bring the star inside cCFB
    # 2. Guide the telescope using a guide probe and offset it to bring the star to the center of cCFB. (You cannot guide on cCFB unless the orientation is constrained later)
    # 3. Once the star is at the center of cCFB of HPF_ACQM, and guiding smoothly on the guide probe. Insert the ACAM and wait for it to stabilise before taking exposure.
    # 4. Take a few exposures, and then retract the ACAM. Make sure star is still at the center of cCFB
    # 5. Repeate the procedure of inserting the ACAM, taking images, retreive,reinsert..
    # 6. Average the centriods to get a good accurate ACAM pixel position of cCFB.
    # 7. Do the same for sCFB and get its pixel coordinates on ACAM.
    
    ### Step 2: Calculate all of the HPF fiber positions in the fplane.txt file
    # 1. Get the latest ACAM pixel positions of BIB, LRS2R, and LRS2B from https://het.as.utexas.edu:8001/trac/WFUCommissioning/wiki/NightOperations/Commissioning/Results
    # 2. Get the fplane.tx coordinates of BIB, LRS2R, and LRS2B from fplane.txt file
    # 3. Fits a linear transformation matrix to transform ACAM pixel coordinates to fplane.txt coordinates
    
    """
    try: 
        coordsOfACAMcCFB = DictOfCoords['ACAMcCFB']
        coordsOfACAMsCFB = DictOfCoords['ACAMsCFB']
        coordsOfBIB = DictOfCoords['BIB']
        coordsOfLRS2R = DictOfCoords['LRS2R']
        coordsOfLRS2B = DictOfCoords['LRS2B']
        fplaneBIB = DictOfCoords['fplaneBIB']
        fplaneLRS2R = DictOfCoords['fplaneLRS2R']
        fplaneLRS2B = DictOfCoords['fplaneLRS2B']
    except KeyError as e:
        print('ERROR: Missing coordinate in input')
        print(e)
        raise


    ##### First doing the calculation for ACAM coordinates
    
    # Telcentric lens image pixel coordinates of central CFB and side CFB as a matrix for transformation
    PixelCoords = np.matrix([[CentroidCoords['cCFB'][0],CentroidCoords['sCFB'][0]],
                             [CentroidCoords['cCFB'][1],CentroidCoords['sCFB'][1]],
                             [1,1]])

    #New ACAM image coordinate measurements of (Central CFB & Side CFB)
    # All other coordinates on ACAM is going to depend on this. So make sure you have best measurment possible.
    ACAMcoords = np.matrix([[coordsOfACAMcCFB[0],coordsOfACAMsCFB[0]],
                            [coordsOfACAMcCFB[1],coordsOfACAMsCFB[1]],
                            [1,1]])

    # Do a least square solution of the TRS transformation from Telecentric image pixel coordinates to ACAM coordinates
    p0 = [0.1,1,1,1]
    pTRS,iter = leastsq(error_funcTRS,p0,args=(PixelCoords,ACAMcoords))
    print('TRS transformation solution from Telecentric pixels to ACAM')
    print('Scale = {0},theta ={1},TranX ={2},TranY={3}'.format(*pTRS))

    RS = np.matmul(RotMatrix(pTRS[1]),ScaleMatrix(pTRS[0]))
    TRS = np.matmul(TranslateMatrix(pTRS[2],pTRS[3]),RS)
    print('Matrix of transformation : {0}'.format(TRS))

    # We chose TRS instead of TRSP since TRS matches the BIB and IMHP orientaiton
    print('*'*20)
    print('New ACAM positions of two CFBs and all HPF fibers')
    print('*'*20)

    ACAMCoordTRS = OrderedDict()
    for fiber in CentroidCoords:
        ACAMCoordTRS[fiber] = np.matmul(TRS,np.matrix([[CentroidCoords[fiber][0]],[CentroidCoords[fiber][1]],[1]]))
        print('{0} : {1:.1f} {2:.1f}'.format(fiber,ACAMCoordTRS[fiber][0,0],ACAMCoordTRS[fiber][1,0]))
        if MakePlot:
            plt.plot(ACAMCoordTRS[fiber][0,0],ACAMCoordTRS[fiber][1,0],'o')
            plt.text(ACAMCoordTRS[fiber][0,0],ACAMCoordTRS[fiber][1,0],fiber+ ' ({0:.1f},{1:.1f})'.format(ACAMCoordTRS[fiber][0,0],ACAMCoordTRS[fiber][1,0]))
    print('*'*20)
    if MakePlot:
        plt.xlabel('X (pixel)')
        plt.ylabel('Y (pixel)')
        plt.title('ACAM pixels')
        plt.show()



    ### ACAM to fplane.txt transformation
    # We shall use BIB, LRS2R, LRS2B to fit the transform.

    # New ACAM measurement of BIB, LRS2R, LRS2B
    ACAMbilrlbCoords = np.matrix([[coordsOfBIB[0],coordsOfLRS2R[0],coordsOfLRS2B[0]],
                                  [coordsOfBIB[1],coordsOfLRS2R[1],coordsOfLRS2B[1]],
                                  [1,1,1]])

    # Latest fplane.txt coordinates of BIB, LRS2R, LRS2B
    SkybilrlbCoords = np.matrix([[fplaneBIB[0],fplaneLRS2R[0],fplaneLRS2B[0]],
                                 [fplaneBIB[1],fplaneLRS2R[1],fplaneLRS2B[1]],
                                 [1,1,1]])

    # Do a least square solution of the TRS transformation from ACAM coordinates to fplane coordinates
    p0 = [0.1,1,1,1]
    pTRS,iter = leastsq(error_funcTRS,p0,args=(ACAMbilrlbCoords,SkybilrlbCoords))
    print('TRS transformation solution from ACAM coordinates to fplane coordinates')
    print('Scale = {0},theta ={1},TranX ={2},TranY={3}'.format(*pTRS))

    RS = np.matmul(RotMatrix(pTRS[1]),ScaleMatrix(pTRS[0]))
    TRS = np.matmul(TranslateMatrix(pTRS[2],pTRS[3]),RS)
    print('Matrix of transformation : {0}'.format(TRS))

    #  Transformation of all the coordinates to fplane.txt Sky coords from ACAM pixels  
    print('Sanity Check of calculating the BIB, LRS2R, LRS2B coorindates in fplane')
    print('TRS: BIB, LRS2R, LRS2B in fplane:', np.matmul(TRS, ACAMbilrlbCoords))
    print('Sanity Check of calculating the cCFB, sCFB coorindates in fplane')
    print('TRS: cCFB, sCFB in fplane:', np.matmul(TRS, ACAMcoords))
    ###############################################
    #  Transformation of all the fiber coordinates
    print('*'*20)
    print('New fplane.txt positions of two CFBs and all HPF fibers')
    print('*'*20)
    SkyCoordTRS = OrderedDict()
    for fiber in ACAMCoordTRS:
        SkyCoordTRS[fiber] = np.matmul(TRS,np.matrix([[ACAMCoordTRS[fiber][0,0]],[ACAMCoordTRS[fiber][1,0]],[1]]))
        print('{0} : {1:.1f} {2:.1f}'.format(fiber,SkyCoordTRS[fiber][0,0],SkyCoordTRS[fiber][1,0]))
        if MakePlot:
            plt.plot(SkyCoordTRS[fiber][0,0],SkyCoordTRS[fiber][1,0],'o')
            plt.text(SkyCoordTRS[fiber][0,0],SkyCoordTRS[fiber][1,0],fiber+ ' ({0:.1f},{1:.1f})'.format(SkyCoordTRS[fiber][0,0],SkyCoordTRS[fiber][1,0]))
    print('*'*20)
    if MakePlot:
        plt.title('fplane.txt')
        plt.xlabel('X (arcsec)')
        plt.ylabel('Y (arcsec)')
        plt.show()
    print('New guider fiducial offset command to go from cCFB to HR Sci fiber is')
    print("""syscmd -T 'Guider1_offset_fiducial(dx_asec={0:.1f},dy_asec={1:.1f},compensate="true")'""".format(SkyCoordTRS['cCFB'][0,0]-SkyCoordTRS['oct3'][0,0],
                                                                                                              SkyCoordTRS['cCFB'][1,0]-SkyCoordTRS['oct3'][1,0]))

    return ACAMCoordTRS, SkyCoordTRS



### Second Scenario is when there is only an over all shift of the entire focal plane array on ACAM. 
# No shifts of the HPF fiber itself with respect to LRS2 or BIB.

def Calculate_new_ACAMcoordinates_from_old(DictOfOldAndNew,DictOfOldPositions,MakePlot=False):
    """
    ## Scenario 2: Update the ACAM coordinates after an IHMP take down and IFU install
    # When: This is when there is only an overall shift to correct after IHMP take down and IFU re-install
    
    # Things to do:
    # 1. Get the old ACAM pixels positions of BIB, LRS2R, and LRS2B
    # 2. Get the latest ACAM pixel positions of BIB, LRS2R, and LRS2B from https://het.as.utexas.edu:8001/trac/WFUCommissioning/wiki/NightOperations/Commissioning/Results
    # 3. Fit a linear transformation from the old pixel coordinates to new pixel coordinates
    # 4. Apply the transformation to old HPF acam pixel positions to obtain new pixel coordinates
    
    """
    try:
        oldBIB = DictOfOldAndNew['oldBIB']
        oldLRS2R = DictOfOldAndNew['oldLRS2R']
        oldLRS2B = DictOfOldAndNew['oldLRS2B']
        newBIB = DictOfOldAndNew['newBIB']
        newLRS2R = DictOfOldAndNew['newLRS2R']
        newLRS2B = DictOfOldAndNew['newLRS2B']
    except KeyError as e:
        print('ERROR: Missing coordinate in input')
        print(e)
        raise


    # Step 1: 
    # Old BIB, LRS2R, LRS2B pixel coordinates in ACAM  
    OldACAM_b_lr_lb_Coords = np.matrix([[oldBIB[0], oldLRS2R[0], oldLRS2B[0]],
                                        [oldBIB[1], oldLRS2R[1], oldLRS2B[1]],
                                        [1,     1,     1]])


    # Step 2:
    # New BIB, LRS2R, LRS2B pixel coordinates in ACAM  
    NewACAM_b_lr_lb_Coords = np.matrix([[newBIB[0], newLRS2R[0], newLRS2B[0]],
                                        [newBIB[1], newLRS2R[1], newLRS2B[1]],
                                        [1,     1,     1]])


    # Step 3:
    # Fit TRS linear transformtion
    p0 = [1,0,0,0]
    pTRS,iter = leastsq(error_funcTRS,p0,args=(OldACAM_b_lr_lb_Coords,NewACAM_b_lr_lb_Coords))
    print('Scale = {0},theta ={1},TranX ={2},TranY={3}'.format(*pTRS))

    print('Transformation matrix')
    RS = np.matmul(RotMatrix(pTRS[1]),ScaleMatrix(pTRS[0]))
    TRS = np.matmul(TranslateMatrix(pTRS[2],pTRS[3]),RS)
    print(TRS)
    print('Residue/Error in the fitted transform (PredictedNewCoords - NewCoords). This is a consistence and sanity check')
    print(np.matmul(TRS,OldACAM_b_lr_lb_Coords)-NewACAM_b_lr_lb_Coords)

    # Step 4
    # Apply the transformation to old coordinates to obtain new ACAM coordinates
    print('*'*20)
    print('New ACAM positions of old ACAM coordinates')
    print('*'*20)
    NewACAMCoordTRS = OrderedDict()
    for fiber in DictOfOldPositions:
        NewACAMCoordTRS[fiber] = np.matmul(TRS,np.matrix([[DictOfOldPositions[fiber][0]],[DictOfOldPositions[fiber][1]],[1]]))
        print('{0} : {1:.1f} {2:.1f}'.format(fiber,NewACAMCoordTRS[fiber][0,0],NewACAMCoordTRS[fiber][1,0]))
        if MakePlot:
            plt.plot(NewACAMCoordTRS[fiber][0,0],NewACAMCoordTRS[fiber][1,0],'o')
            plt.text(NewACAMCoordTRS[fiber][0,0],NewACAMCoordTRS[fiber][1,0],fiber+ ' ({0:.1f},{1:.1f})'.format(NewACAMCoordTRS[fiber][0,0],NewACAMCoordTRS[fiber][1,0]))
    print('*'*20)
    if MakePlot:
        plt.title('New ACAM coords')
        plt.xlabel('X (pixels)')
        plt.ylabel('Y (pixels)')
        plt.show()

    return NewACAMCoordTRS

#################### Third Scenrario
# fplane positions of the Central CFB and Side CFB are calculated using simultanious measurments with VIRUS
# We need to transform HPF fibers also to the fplane coordinates using the Lab measurment

def Calculate_all_fplanecoordinates_from_CFBs(DictOfCFBCoords,MakePlot=False):
    """ 
    Scenrio 3
    Calculates all the coordinates from scratch using new measured positions of central and side CFBs
    ## Scenario 3: Fibers based on cCFB and sCFB positions ############################################
    
    # When: This is when you have new measurmenets of the HPF's cetranl as sied CFBs in the HET's focal plane or ACAM
    
    # Things to do: 
    # 1. Measure the cCFB and sCFB positions in fplane positions using VIRUS unit or in ACAM
    # 2. Calculate all of the HPF fiber positions in the fplane.txt file
    # 3. Calculate the offsets in X & Y coordinates for moving from cCFB to science fibers.
    
    ### Step 1: Measure the cCFB and sCFB positions in fplane
    ### Step 2: Calculate all of the HPF fiber positions in the fplane.txt file
    #  Fits a linear transformation matrix to transform PSU lab measurments coordinates to fplane.txt coordinates
    
    """
    try: 
        coordsOfcCFB = DictOfCFBCoords['cCFB']
        coordsOfsCFB = DictOfCFBCoords['sCFB']
    except KeyError as e:
        print('ERROR: Missing coordinate in input')
        print(e)
        raise


    ##### First doing the calculation for fplane coordinates
    
    # PSU Telcentric lens image pixel coordinates of central CFB and side CFB as a matrix for transformation
    PixelCoords = np.matrix([[CentroidCoords['cCFB'][0],CentroidCoords['sCFB'][0]],
                             [CentroidCoords['cCFB'][1],CentroidCoords['sCFB'][1]],
                             [1,1]])

    # New fplane/ACAM coordinate measurements of (Central CFB & Side CFB)
    # All other coordinates on ACAM is going to depend on this. So make sure you have best measurment possible.
    fplanecoords = np.matrix([[coordsOfcCFB[0],coordsOfsCFB[0]],
                              [coordsOfcCFB[1],coordsOfsCFB[1]],
                              [1,1]])

    # Do a least square solution of the TRS transformation from Telecentric image pixel coordinates to fplane/ACAM coordinates
    p0 = [0.1,1,1,1]
    pTRS,iter = leastsq(error_funcTRS,p0,args=(PixelCoords,fplanecoords))
    print('TRS transformation solution from Telecentric pixels to fplane')
    print('Scale = {0},theta ={1},TranX ={2},TranY={3}'.format(*pTRS))

    RS = np.matmul(RotMatrix(pTRS[1]),ScaleMatrix(pTRS[0]))
    TRS = np.matmul(TranslateMatrix(pTRS[2],pTRS[3]),RS)
    print('Matrix of transformation : {0}'.format(TRS))

    # We chose TRS instead of TRSP since TRS matches the fplane orientaiton to PSU telecentric coordinates
    print('*'*20)
    print('New fplane positions of two CFBs and all HPF fibers')
    print('*'*20)

    fplaneCoordTRS = OrderedDict()
    for fiber in CentroidCoords:
        fplaneCoordTRS[fiber] = np.matmul(TRS,np.matrix([[CentroidCoords[fiber][0]],[CentroidCoords[fiber][1]],[1]]))
        print('{0} : {1:.1f} {2:.1f}'.format(fiber,fplaneCoordTRS[fiber][0,0],fplaneCoordTRS[fiber][1,0]))
        if MakePlot:
            plt.plot(fplaneCoordTRS[fiber][0,0],fplaneCoordTRS[fiber][1,0],'o')
            plt.text(fplaneCoordTRS[fiber][0,0],fplaneCoordTRS[fiber][1,0],fiber+ ' ({0:.1f},{1:.1f})'.format(fplaneCoordTRS[fiber][0,0],fplaneCoordTRS[fiber][1,0]))
    print('*'*20)
    if MakePlot:
        plt.title('fplane.txt')
        plt.xlabel('X (arcsec)')
        plt.ylabel('Y (arcsec)')
        plt.show()


    print('New guider fiducial offset command to go from cCFB to HR Sci fiber is')
    print("""syscmd -T 'Guider1_offset_fiducial(dx_asec={0:.1f},dy_asec={1:.1f},compensate="true")'""".format(fplaneCoordTRS['cCFB'][0,0]-fplaneCoordTRS['oct3'][0,0],
                                                                                                              fplaneCoordTRS['cCFB'][1,0]-fplaneCoordTRS['oct3'][1,0]))

    return fplaneCoordTRS



#####################################################################################################
def LoadInputCoordsFile(inpfilename):
    """ Loads and returns the coordinates in the input file as a dictionary.
    All blank lines and comments starting with # are ignored.
    Inlines comments startting with # are ignored.
    Valid line format is as follows
    FiberName = Xcoord , Ycoords  # Any optional inline comment which will be ignored
    """
    CoordsDic = OrderedDict()
    with open(inpfilename,'r') as inpfile:
        for line in inpfile:
            line = line.rstrip()
            if (len(line) < 1) or (line[0] =='#'):  # Ignore blank lines and comment lines starting with #
                continue
            line = line.split('#')[0]  # Ignore inline comments after #
            name = line.split('=')[0].strip()
            value = tuple(float(c.strip()) for c in line.split('=')[1].strip().split(','))
            CoordsDic[name]=value
    return CoordsDic
            
def parse_args():
    """ Parses the command line input arguments"""
    parser = argparse.ArgumentParser(description="Script to calculate the HPFs fiber and CFB positions on HET telescope.")
    parser.add_argument('--ShowPlots', action='store_true', help='Show Plots')
    subparsers = parser.add_subparsers(help='Choose one of the subcommands below and add -h flag for more help')

    parser_a = subparsers.add_parser('fromscratch', help='Calculate both ACAM positions as well as fplane positions using cCFB and sCFB')
    parser_a.add_argument('MeasuredCoordsFile', type=str,
                        help="Input Coords file containing the New ACAM image coordinate measurements of (Central CFB & Side CFB), BIB, LRS2R, LRS2B, and Latest fplane.txt coordinates of BIB, LRS2R, LRS2B. See the example file for the keywords and format to use.")
    parser_a.set_defaults(command='fromscratch')

    parser_b = subparsers.add_parser('old2newacam', help='Calculate new ACAM positions based on old ACAM positions')
    parser_b.add_argument('OldAndNewCoordsFile', type=str,
                        help="Input Coords file containing the Old BIB, LRS2R, LRS2B pixel coordinates in ACAM as well as new BIB, LRS2R, LRS2B pixel coordinates in ACAM.  See the example file for the keywords and format to use.")
    parser_b.add_argument('OldCoordsToRecalculate', type=str,
                        help="Input Coords file containing the Old pixel coordinates of HPF fibers in ACAM which needs to be recalculated. See the example file for the format to use.")
    parser_b.set_defaults(command='old2newacam')

    parser_c = subparsers.add_parser('newfplane', help='Calculate new fplane positions based on new fplane position measurement of cCFB and sCFB')
    parser_c.add_argument('MeasuredCFBfplaneCoordsFile', type=str,
                          help="Input Coords file containing the new fplane coordinate measurements of Central CFB  (cCFB) & Side CFB (cCFB).  See the example file for the keywords and format to use.")
    parser_c.set_defaults(command='newfplane')

    args = parser.parse_args()
    return args
    

def main():
    """ Standalone Script to calculate new HPF fiber positions on HET focal plan or ACAM"""
    # Parse input arguments
    args = parse_args()
    if args.command == 'fromscratch':
        DictOfCoords = LoadInputCoordsFile(args.MeasuredCoordsFile)
        _ = Calculate_all_coordinates_from_scratch(DictOfCoords,MakePlot=args.ShowPlots)
        
    elif args.command == 'old2newacam':
        DictOfOldAndNew = LoadInputCoordsFile(args.OldAndNewCoordsFile)
        DictOfOldPositions = LoadInputCoordsFile(args.OldCoordsToRecalculate)
        _ = Calculate_new_ACAMcoordinates_from_old(DictOfOldAndNew,DictOfOldPositions,MakePlot=args.ShowPlots)

    elif args.command == 'newfplane':
        DictOfCFBCoords = LoadInputCoordsFile(args.MeasuredCFBfplaneCoordsFile)
        _ = Calculate_all_fplanecoordinates_from_CFBs(DictOfCFBCoords,MakePlot=args.ShowPlots)
        
    print('-'*20)
    

if __name__ == '__main__':
    main()
